#include <bindings.dsl.h>
#include <apr_file_io.h>

module Bindings.APR.File.IO where
#strict_import
import Bindings.APR
import Bindings.APR.ErrNo
import Bindings.APR.File.Info
import Bindings.APR.File.Types
import Bindings.APR.Pools
import Bindings.APR.Time
import Bindings.APR.Want

#num APR_FOPEN_READ
#num APR_FOPEN_WRITE
#num APR_FOPEN_CREATE
#num APR_FOPEN_APPEND
#num APR_FOPEN_TRUNCATE
#num APR_FOPEN_BINARY
#num APR_FOPEN_EXCL
#num APR_FOPEN_BUFFERED
#num APR_FOPEN_DELONCLOSE
#num APR_FOPEN_XTHREAD
#num APR_FOPEN_SHARELOCK
#num APR_FOPEN_NOCLEANUP
#num APR_FOPEN_SENDFILE_ENABLED
#num APR_FOPEN_LARGEFILE
#num APR_FOPEN_SPARSE

#num APR_READ
#num APR_WRITE
#num APR_CREATE
#num APR_APPEND
#num APR_TRUNCATE
#num APR_BINARY
#num APR_EXCL
#num APR_BUFFERED
#num APR_DELONCLOSE
#num APR_XTHREAD
#num APR_SHARELOCK
#num APR_FILE_NOCLEANUP
#num APR_SENDFILE_ENABLED
#num APR_LARGEFILE

#num APR_SET
#num APR_CUR
#num APR_END

#num APR_FILE_ATTR_READONLY
#num APR_FILE_ATTR_EXECUTABLE
#num APR_FILE_ATTR_HIDDEN

#num APR_MAX_IOVEC_SIZE

#integral_t apr_fileattrs_t
#integral_t apr_seek_where_t

#num APR_FLOCK_SHARED
#num APR_FLOCK_EXCLUSIVE
#num APR_FLOCK_TYPEMASK
#num APR_FLOCK_NONBLOCK

#ccall apr_file_open  , Ptr (Ptr <apr_file_t>) -> Ptr CChar -> <apr_int32_t> -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_close , Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_remove, Ptr CChar -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_rename, Ptr CChar -> Ptr CChar -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_copy  , Ptr CChar -> Ptr CChar -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_append, Ptr CChar -> Ptr CChar -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_eof   , Ptr <apr_file_t> -> IO <apr_status_t>

#ccall apr_file_open_stderr, Ptr (Ptr <apr_file_t>) -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_open_stdout, Ptr (Ptr <apr_file_t>) -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_open_stdin , Ptr (Ptr <apr_file_t>) -> Ptr <apr_pool_t> -> IO <apr_status_t>

#ccall apr_file_open_flags_stderr, Ptr (Ptr <apr_file_t>) -> <apr_int32_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_open_flags_stdout, Ptr (Ptr <apr_file_t>) -> <apr_int32_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_open_flags_stdin , Ptr (Ptr <apr_file_t>) -> <apr_int32_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>

#ccall apr_file_read       , Ptr <apr_file_t> -> Ptr () -> Ptr <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_write      , Ptr <apr_file_t> -> Ptr () -> Ptr <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_writev     , Ptr <apr_file_t> -> Ptr <iovec> -> <apr_size_t> -> Ptr <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_read_full  , Ptr <apr_file_t> -> Ptr () -> <apr_size_t> -> Ptr <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_write_full , Ptr <apr_file_t> -> Ptr () -> <apr_size_t> -> Ptr <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_writev_full, Ptr <apr_file_t> -> Ptr <iovec> -> <apr_size_t> -> Ptr <apr_size_t> -> IO <apr_status_t>

#ccall apr_file_putc    , CChar -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_getc    , CChar -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_ungetc  , CChar -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_gets    , Ptr CChar -> CInt -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_puts    , Ptr CChar -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_flush   , Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_dup     , Ptr (Ptr <apr_file_t>) -> Ptr <apr_file_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_dup2    , Ptr <apr_file_t> -> Ptr <apr_file_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_setaside, Ptr (Ptr <apr_file_t>) -> Ptr <apr_file_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_buffer_set      , Ptr <apr_file_t> -> Ptr CChar -> <apr_size_t> -> IO <apr_status_t>
#ccall apr_file_buffer_size_get , Ptr <apr_size_t> -> IO <apr_size_t>
#ccall apr_file_seek            , Ptr <apr_file_t> -> <apr_seek_where_t> -> Ptr <apr_off_t> -> IO <apr_status_t>
#ccall apr_file_pipe_create     , Ptr (Ptr <apr_file_t>) -> Ptr (Ptr <apr_file_t>) -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_pipe_create_ex  , Ptr (Ptr <apr_file_t>) -> Ptr (Ptr <apr_file_t>) -> <apr_int32_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_namedpipe_create, Ptr CChar -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_pipe_timeout_get, Ptr <apr_file_t> -> Ptr <apr_interval_time_t> -> IO <apr_status_t>
#ccall apr_file_pipe_timeout_set, Ptr <apr_file_t> -> <apr_interval_time_t> -> IO <apr_status_t>
#ccall apr_file_lock     , Ptr <apr_file_t> -> CInt -> IO <apr_status_t>
#ccall apr_file_unlock   , Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_name_get , Ptr (Ptr CChar) -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_data_get , Ptr (Ptr ()) -> Ptr CChar -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_data_set , Ptr <apr_file_t> -> Ptr () -> Ptr CChar -> FunPtr (Ptr () -> IO <apr_status_t>) -> IO <apr_status_t>
-- apr_file_printf can't be wrapped as it takes varargs.
#ccall apr_file_perms_set, Ptr CChar -> <apr_fileperms_t> -> IO <apr_status_t>
#ccall apr_file_attrs_set, Ptr CChar -> <apr_fileattrs_t> -> <apr_fileattrs_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_mtime_set, Ptr CChar -> <apr_time_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_dir_make      , Ptr CChar -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_dir_make_recursive, Ptr CChar -> <apr_fileperms_t> -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_dir_remove    , Ptr CChar -> Ptr <apr_pool_t> -> IO <apr_status_t>
#ccall apr_file_info_get , Ptr <apr_finfo_t> -> <apr_int32_t> -> Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_trunc    , Ptr <apr_file_t> -> <apr_off_t> -> IO <apr_status_t>
#ccall apr_file_flags_get, Ptr <apr_file_t> -> IO <apr_int32_t>

#ccall apr_file_pool_get     , Ptr <apr_file_t> -> IO (Ptr <apr_pool_t>)
#ccall apr_file_inherit_set  , Ptr <apr_file_t> -> IO <apr_status_t>
#ccall apr_file_inherit_unset, Ptr <apr_file_t> -> IO <apr_status_t>
